package cn.iocoder.yudao.module.system.job.holiday;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.tenant.core.job.TenantJob;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaybalancesetting.HolidayBalanceSettingDO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaysetting.HolidaySettingDO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaysettingrange.HolidaySettingRangeDO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidayworkingage.HolidayWorkingAgeDO;
import cn.iocoder.yudao.module.system.service.holiday.holidaybalancesetting.HolidayBalanceSettingService;
import cn.iocoder.yudao.module.system.service.holiday.holidaysetting.HolidaySettingService;
import cn.iocoder.yudao.module.system.service.holiday.holidaysettingrange.HolidaySettingRangeService;
import cn.iocoder.yudao.module.system.service.holiday.holidayuserrecord.HolidayUserRecordService;
import cn.iocoder.yudao.module.system.service.holiday.holidayworkingage.HolidayWorkingAgeService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
@Slf4j
public class HolidayGrantJob {

    // TODO: 2024/4/22  - 每十分钟执行一次 将漏打卡的设为缺卡 这里的update要保证命中索引 保证是行锁 不然容易导致锁表

    @Resource
    private HolidayBalanceSettingService holidayBalanceSettingService;
    @Resource
    private HolidaySettingService holidaySettingService;
    @Resource
    private HolidayUserRecordService holidayUserRecordService;
    @Resource
    private HolidaySettingRangeService holidaySettingRangeService;
    @Resource
    private HolidayWorkingAgeService holidayWorkingAgeService;

    @XxlJob("holidayGrantJob")
    @TenantJob // --- ⚠️ 这个注解 会将租户列表拉出来 完了后逐个租户执行 定时任务需要注意
    public ReturnT<String> execute() throws Exception {
        // -- 发放假期定时任务 - 每天凌晨3点执行一次 判断下是不是该发放了
        log.info("开始 发放假期");
        // -- 先获取所有假期余额设置
        LocalDateTime now = LocalDateTime.now();
        List<HolidayBalanceSettingDO> list = holidayBalanceSettingService.list(new LambdaQueryWrapper<HolidayBalanceSettingDO>()
                .eq(HolidayBalanceSettingDO::getStatus, 1)
                .in(HolidayBalanceSettingDO::getType, Arrays.asList(1, 2)));
        if (CollectionUtil.isEmpty(list)) {
            log.info("结束 发放假期");
            return ReturnT.SUCCESS;
        }

        List<Long> holidaySettingIds = list.stream().map(HolidayBalanceSettingDO::getHolidaySettingId).collect(Collectors.toList());
        List<HolidaySettingDO> holidaySettingDOS = holidaySettingService.listByIds(holidaySettingIds);
        Map<Long, HolidaySettingDO> holidaySettingMap = holidaySettingDOS.stream().collect(Collectors.toMap(HolidaySettingDO::getId, holidaySettingDO -> holidaySettingDO));

        List<HolidaySettingRangeDO> holidaySettingRangeDOS = holidaySettingRangeService.getListByHolidaySettingIds(holidaySettingIds);
        Map<Long, List<HolidaySettingRangeDO>> holidaySettingRangeMap = holidaySettingRangeDOS.stream().collect(Collectors.groupingBy(HolidaySettingRangeDO::getHolidaySettingId, Collectors.toList()));

        List<HolidayWorkingAgeDO> holidayWorkingAgeDOS = holidayWorkingAgeService.selectBySettingIds(holidaySettingIds);
        Map<Long, List<HolidayWorkingAgeDO>> holidayWorkingAgeDOMap = holidayWorkingAgeDOS.stream().collect(Collectors.groupingBy(HolidayWorkingAgeDO::getHolidaySettingId, Collectors.toList()));

        List<HolidayBalanceSettingDO> editList = new ArrayList<>();
        // -- 过滤出 按月发放 or 按每年一月一号 发放 的并且时间到了的
        List<HolidayBalanceSettingDO> fixedGrantList = list.stream().filter(holidayBalanceSettingDO ->
                (holidayBalanceSettingDO.getType() == 1 || (holidayBalanceSettingDO.getType() == 2 && holidayBalanceSettingDO.getIssueTimeType() == 2))
                        && holidayBalanceSettingDO.getIssueTime() != null
                        && holidayBalanceSettingDO.getIssueTime().isBefore(now)).collect(Collectors.toList());

        if (CollectionUtil.isEmpty(fixedGrantList)) {
            for (HolidayBalanceSettingDO holidayBalanceSettingDO : fixedGrantList) {
                // 计算下一次发放日期
                LocalDateTime issueTime = holidaySettingService.calculateNextReleaseDate(now, holidayBalanceSettingDO);
                holidayBalanceSettingDO.setIssueTime(issueTime);
                editList.add(holidayBalanceSettingDO);
                Long holidaySettingId = holidayBalanceSettingDO.getHolidaySettingId();
                // 发放假期
                holidayUserRecordService.grant(holidaySettingMap.get(holidaySettingId), holidaySettingRangeMap.get(holidaySettingId), holidayBalanceSettingDO, holidayWorkingAgeDOMap.get(holidaySettingId));
            }
            holidayBalanceSettingService.batchUpdate(editList);
        }

        // TODO: 2024/10/23  这里要怎么去判断按照用户入职时间的话 是否发放过？
        // -- 过滤出按照员工入职时间发放的
//        List<HolidayBalanceSettingDO> employmentGrantList = list.stream().filter(holidayBalanceSettingDO ->
//                        holidayBalanceSettingDO.getType() == 2 && holidayBalanceSettingDO.getIssueTimeType() == 1)
//                .collect(Collectors.toList());
//        // -- 获取到这些后 - 再获取到用户
//        if (CollectionUtil.isNotEmpty(employmentGrantList)) {
//            for (HolidayBalanceSettingDO holidayBalanceSettingDO : employmentGrantList) {
//                Long holidaySettingId = holidayBalanceSettingDO.getHolidaySettingId();
//                List<HolidaySettingRangeDO> rangeList = holidaySettingRangeMap.get(holidaySettingId);
//                List<AdminUserDO> usersByRange = holidaySettingRangeService.getUsersByRange(rangeList);
//                // 搞个表记录下按照员工入职时间发放的
//                List<AdminUserDO> users = usersByRange.stream().filter(a -> a.getEntryDate() != null && a.getEntryDate().isBefore(now)).collect(Collectors.toList());
//                if (CollectionUtil.isNotEmpty(users)) {
//                    holidayUserRecordService.grant(holidaySettingMap.get(holidaySettingId), holidaySettingRangeMap.get(holidaySettingId), holidayBalanceSettingDO, holidayWorkingAgeDOMap.get(holidaySettingId));
//                }
//            }
//        }

        log.info("结束 发放假期");
        // 返回执行成功
        return ReturnT.SUCCESS;
    }

}
